@@ -143,34 +143,21 @@ For all the multi file includes, the files are simply appended using a newline
 character.
 
 
-To allow for job templates to perform substitution on the path names, when a
-filename containing a python format placeholder is encountered, lazy loading
-support is enabled, where instead of returning the contents back during yaml
-parsing, it is delayed until the variable substitution is performed.
+You can also use variables in included file paths.
 
 Example:
 
     .. literalinclude:: /../../tests/yamlparser/job_fixtures/lazy-load-jobs001.yaml
 
+    with variable substitution inside included files:
+
+    .. literalinclude:: /../../tests/yamlparser/job_fixtures/lazy-load-with-variables.yaml
+
     using a list of files:
 
     .. literalinclude::
         /../../tests/yamlparser/job_fixtures/lazy-load-jobs-multi001.yaml
 
-.. note::
-
-    Because lazy-loading involves performing the substitution on the file
-    name, it means that jenkins-job-builder can not call the variable
-    substitution on the contents of the file. This means that the
-    ``!include-raw:`` tag will behave as though ``!include-raw-escape:`` tag
-    was used instead whenever name substitution on the filename is to be
-    performed.
-
-    Given the behaviour described above, when substitution is to be performed
-    on any filename passed via ``!include-raw-escape:`` the tag will be
-    automatically converted to ``!include-raw:`` and no escaping will be
-    performed.
-
 
 The tag ``!include-jinja2:`` will treat the given string or list of strings as
 filenames to be opened as Jinja2 templates, which should be rendered to a
@@ -214,7 +201,6 @@ Examples:
 """
 
 import abc
-import os.path
 import logging
 import traceback
 import sys
@@ -262,10 +248,10 @@ class BaseYamlObject(metaclass=abc.ABCMeta):
         return cls(loader.jjb_config, loader, loader.pos_from_node(node), value)
 
     def __init__(self, jjb_config, loader, pos):
-        self._search_path = jjb_config.yamlparser["include_path"]
-        if loader.source_path:
+        self._search_path = jjb_config.yamlparser["include_path"].copy()
+        if loader.source_dir:
             # Loaded from a file, find includes beside it too.
-            self._search_path.append(os.path.dirname(loader.source_path))
+            self._search_path.append(loader.source_dir)
         self._loader = loader
         self._pos = pos
         allow_empty = jjb_config.yamlparser["allow_empty_variables"]
@@ -370,7 +356,9 @@ class J2Yaml(J2Template):
 
     def expand(self, expander, params):
         text = self._render(params)
-        data = self._loader.load(text, source_path="<expanded j2-yaml>")
+        data = self._loader.load(
+            text, source_path="<expanded j2-yaml>", source_dir=self._loader.source_dir
+        )
         try:
             return expander.expand(data, params)
         except JenkinsJobsException as x:
